@@ -53,7 +53,7 @@ class _Nothing:
 
     def __new__(cls) -> _Nothing:
         if _Nothing._singleton is None:
-            _Nothing._singleton = super(_Nothing, cls).__new__(cls)
+            _Nothing._singleton = super().__new__(cls)
         return _Nothing._singleton
 
     def __repr__(self) -> str:
@@ -123,7 +123,7 @@ def _retry_eval_with_missing_locals(
     # We use a value here instead of a "while True" to not get the program stuck in an endless loop.
     for _ in range(100):
         try:
-            val = eval(expression, globalns, localns)  # noqa: eval-used
+            val = eval(expression, globalns, localns)  # noqa: PGH001
             break
         except NameError as e:
             missing = str(e).split("'")[1]
@@ -184,8 +184,8 @@ def _custom_get_type_hints(cls: Type[_BaseTpcpObject]) -> Dict[str, Any]:
             if value is None:
                 value = type(None)
             elif isinstance(value, str):
-                # TODO: This does not check if the str is a valid expression.
-                #   This might not be an issue, but could lead to obscure error messages.
+                # NOTE: This does not check if the str is a valid expression.
+                #       This might not be an issue, but could lead to obscure error messages.
                 value = _retry_eval_with_missing_locals(value, base_globals)
             hints[name] = value
     return hints
@@ -273,15 +273,8 @@ class _BaseTpcpObject:
         cls.__field_annotations_cache__ = field_annotations
         return field_annotations
 
-    def __init_subclass__(cls, *, _skip_validation: bool = False, **kwargs: Any):
+    def __init_subclass__(cls, **kwargs: Any):
         super().__init_subclass__(**kwargs)
-        # Todo: Deprecate this parameter
-        if _skip_validation:
-            warnings.warn(
-                "`_skip_validation` is deprecated and has no effect."
-                "Validation is now performed when the class is actually used and hence has no runtime cost "
-                "during class definition."
-            )
         # If the class has no init or did inherit its init from the parent (i.e. it is not in its own dict), there is
         # nothing to do.
         if "__init__" not in cls.__dict__:
@@ -294,7 +287,7 @@ class _BaseTpcpObject:
 class BaseTpcpObject(_BaseTpcpObject):
     """Baseclass for all tpcp objects."""
 
-    _composite_params: ClassVar[Tuple[str, ...]] = tuple()
+    _composite_params: ClassVar[Tuple[str, ...]] = ()
 
     def get_params(self, deep: bool = True) -> Dict[str, Any]:
         """Get parameters for this algorithm.
@@ -364,7 +357,7 @@ class BaseTpcpObject(_BaseTpcpObject):
         return "".join(result) + ")"
 
     @classmethod
-    def __clone_param__(cls, param_name: str, value: Any) -> Any:  # noqa: unused-argument
+    def __clone_param__(cls, param_name: str, value: Any) -> Any:  # pylint: disable=unused-argument
         """Handle cloning of specific parameters of the object.
 
         This method exists only to allow object the handling of "strange" python datatypes that can not be deepcopied
@@ -420,7 +413,7 @@ def _get_params(instance: _BaseTpcpObject, deep: bool = True) -> Dict[str, Any]:
         _set_tpcp_validated(instance, True)
 
     valid_fields = get_param_names(type(instance))
-    comp_fields = getattr(instance, "_composite_params", tuple())
+    comp_fields = getattr(instance, "_composite_params", ())
     out: Dict[str, Any] = {}
     for key in valid_fields:
         value = getattr(instance, key)
@@ -458,10 +451,7 @@ def _set_comp_field(instance, field_name, params):
     comp_list = list(getattr(instance, field_name))
     for key, old_value in comp_list:
         nested_values = comp_params.pop(key, {})
-        if "*" not in nested_values:
-            new_value = old_value
-        else:
-            new_value = nested_values.pop("*")
+        new_value = old_value if "*" not in nested_values else nested_values.pop("*")
         if nested_values:
             new_value = new_value.set_params(**nested_values)
         new_list.append((key, new_value))
@@ -488,7 +478,7 @@ def _set_params(instance: BaseTpcpObjectObjT, **params: Any) -> BaseTpcpObjectOb
     if not params:
         return instance
     valid_params = instance.get_params(deep=True)
-    comp_fields = getattr(instance, "_composite_params", tuple())
+    comp_fields = getattr(instance, "_composite_params", ())
 
     nested_params: DefaultDict[str, Any] = defaultdict(dict)  # grouped by prefix
     for key, value in params.items():
@@ -641,7 +631,7 @@ def clone(algorithm: T, *, safe: bool = False) -> T:
                   This clone method, will deepcopy sklearn models.
                   This means fitted models will be copied and are still available afterwards.
                   For more information have a look at the documentation about "inputs and results" in tpcp.
-                  TODO: Link
+                  (see :ref:`general concepts <general_concepts>` and :ref:`sklearn differences <sklearn_differences>`)
 
     Parameters
     ----------
@@ -655,10 +645,10 @@ def clone(algorithm: T, *, safe: bool = False) -> T:
         return algorithm
     # Handle named tuple
     if isinstance(algorithm, tuple) and hasattr(algorithm, "_asdict") and hasattr(algorithm, "_fields"):
-        return type(algorithm)(*(clone(a, safe=safe) for a in algorithm))  # noqa: to-many-function-args
+        return type(algorithm)(*(clone(a, safe=safe) for a in algorithm))
     # XXX: not handling dictionaries
     if isinstance(algorithm, (list, tuple, set, frozenset)):
-        return type(algorithm)([clone(a, safe=safe) for a in algorithm])  # noqa: to-many-function-args
+        return type(algorithm)([clone(a, safe=safe) for a in algorithm])
     # Compared to sklearn, we check specifically for BaseTpcpObject and not just if `get_params` is defined on the
     # object.
     # Due to the way algorithms/pipelines in tpcp work, they need to inherit from BaseTpcpObject.
@@ -732,9 +722,9 @@ class CloneFactory(BaseFactory, Generic[T]):
 # Ideally clone factory would be some generic type of proxy, but it isn't.
 # Therefore, we fake the return type, so that external type checking in user code works without any errors.
 # As long as `cf` is used for its intended purpose that should not matter much.
-def cf(default_value: T) -> T:  # noqa: invalid-name
+def cf(default_value: T) -> T:  # pylint: disable=invalid-name
     """Wrap mutable default value with the `CloneFactory`.
 
     This is basically an alias for :class:`~tpcp.CloneFactory`
     """
-    return CloneFactory(default_value)  # type: ignore
+    return CloneFactory(default_value)  # type: ignore  # noqa: PGH003
