@@ -1,5 +1,4 @@
 """Base class for all datasets."""
-import dataclasses
 from typing import Dict, Iterator, List, Optional, Sequence, Tuple, TypeVar, Union, cast, overload
 
 import numpy as np
@@ -148,15 +147,9 @@ class _Dataset(BaseTpcpObject):
             New dataset object filtered by specified parameters.
 
         """
-        if (
-            list(
-                map(
-                    lambda x: x is None or (isinstance(x, dict) and len(x) == 0),
-                    (groups, index, bool_map, kwargs),
-                )
-            ).count(False)
-            > 1
-        ):
+        if [x is None or (isinstance(x, dict) and len(x) == 0) for x in [groups, index, bool_map, kwargs]].count(
+            False
+        ) > 1:
             raise ValueError("Only one of `groups`, `selected_keys`, `index`, `bool_map` or kwarg can be set!")
 
         if groups is not None:
@@ -193,19 +186,13 @@ class _Dataset(BaseTpcpObject):
 
     def __repr__(self) -> str:
         """Return string representation of the dataset object."""
-        if self.groupby_cols is None:
-            repr_index = self.index
-        else:
-            repr_index = self.grouped_index
+        repr_index = self.index if self.groupby_cols is None else self.grouped_index
         repr_index = str(repr_index).replace("\n", "\n   ")
         return f"{self.__class__.__name__} [{self.shape[0]} groups/rows]\n\n   {repr_index}\n\n   "[:-5]
 
     def _repr_html_(self) -> str:
         """Return html representation of the dataset object."""
-        if self.groupby_cols is None:
-            repr_index = self.index
-        else:
-            repr_index = self.grouped_index
+        repr_index = self.index if self.groupby_cols is None else self.grouped_index
 
         df_repr = (
             repr_index._repr_html_()
@@ -313,7 +300,7 @@ class _Dataset(BaseTpcpObject):
                 f" only a single group left in a data subset. " + group_error_str
             )
 
-    def create_group_labels(self, label_cols: Union[str, List[str]]):
+    def create_group_labels(self, label_cols: Union[str, List[str]]) -> List[str]:
         """Generate a list of labels for each group/row in the dataset.
 
         .. note::
@@ -334,7 +321,7 @@ class _Dataset(BaseTpcpObject):
         """
         unique_index = self._get_unique_groups().to_frame()
         try:
-            return unique_index.set_index(label_cols).index.to_list()
+            return [str(g) for g in unique_index.set_index(label_cols).index.to_list()]
         except KeyError as e:
             if self.groupby_cols is not None:
                 raise KeyError(
@@ -511,6 +498,7 @@ class Dataset(_Dataset):
 
     If we want to use datasets in combination with :class:`~sklearn.model_selection.GroupKFold`, we can generate
     valid group labels as follows.
+    These grouplabels are strings representing the unique value of the index at the specified levels.
 
     .. note::
         You usually don't want to use that in combination with `self.groupby`.
@@ -518,19 +506,19 @@ class Dataset(_Dataset):
     >>> # We are using the ungrouped dataset again!
     >>> group_labels = dataset.create_group_labels(["patient", "test"])
     >>> pd.concat([dataset.index, pd.Series(group_labels, name="group_labels")], axis=1)
-          patient    test extra         group_labels
-    0   patient_1  test_1     1  (patient_1, test_1)
-    1   patient_1  test_1     2  (patient_1, test_1)
-    2   patient_1  test_2     1  (patient_1, test_2)
-    3   patient_1  test_2     2  (patient_1, test_2)
-    4   patient_2  test_1     1  (patient_2, test_1)
-    5   patient_2  test_1     2  (patient_2, test_1)
-    6   patient_2  test_2     1  (patient_2, test_2)
-    7   patient_2  test_2     2  (patient_2, test_2)
-    8   patient_3  test_1     1  (patient_3, test_1)
-    9   patient_3  test_1     2  (patient_3, test_1)
-    10  patient_3  test_2     1  (patient_3, test_2)
-    11  patient_3  test_2     2  (patient_3, test_2)
+          patient    test extra             group_labels
+    0   patient_1  test_1     1  ('patient_1', 'test_1')
+    1   patient_1  test_1     2  ('patient_1', 'test_1')
+    2   patient_1  test_2     1  ('patient_1', 'test_2')
+    3   patient_1  test_2     2  ('patient_1', 'test_2')
+    4   patient_2  test_1     1  ('patient_2', 'test_1')
+    5   patient_2  test_1     2  ('patient_2', 'test_1')
+    6   patient_2  test_2     1  ('patient_2', 'test_2')
+    7   patient_2  test_2     2  ('patient_2', 'test_2')
+    8   patient_3  test_1     1  ('patient_3', 'test_1')
+    9   patient_3  test_1     2  ('patient_3', 'test_1')
+    10  patient_3  test_2     1  ('patient_3', 'test_2')
+    11  patient_3  test_2     2  ('patient_3', 'test_2')
 
     """
 
@@ -546,6 +534,7 @@ class Dataset(_Dataset):
     @staticmethod
     def as_dataclass():
         """Return a version of the Dataset class that can be subclassed using dataclasses."""
+        import dataclasses  # pylint: disable=import-outside-toplevel
 
         @dataclasses.dataclass(eq=False, repr=False, order=False)
         class DatasetDc(_Dataset):
@@ -562,7 +551,7 @@ class Dataset(_Dataset):
 
         Note, this requires `attrs` to be installed!
         """
-        from attrs import define  # noqa: import-outside-toplevel
+        from attrs import define  # pylint: disable=import-outside-toplevel
 
         @define(eq=False, repr=False, order=False, kw_only=True, slots=False)
         class DatasetAt(_Dataset):
