@@ -69,10 +69,10 @@ class DummyOptimize(BaseOptimize[PipelineT, DatasetT]):
 
     optimized_pipeline_: PipelineT
 
-    def __init__(self, pipeline: PipelineT) -> None:  # noqa: super-init-not-called
+    def __init__(self, pipeline: PipelineT) -> None:
         self.pipeline = pipeline
 
-    def optimize(self, dataset: DatasetT, **optimize_params: Any) -> Self:
+    def optimize(self, dataset: DatasetT, **optimize_params: Any) -> Self:  # noqa: ARG002
         """Run the "dummy" optimization.
 
         Parameters
@@ -150,7 +150,7 @@ class Optimize(BaseOptimize[OptimizablePipelineT, DatasetT]):
     optimized_pipeline_: OptimizablePipelineT
     optimization_info_: Any
 
-    def __init__(  # noqa: super-init-not-called
+    def __init__(
         self, pipeline: OptimizablePipelineT, *, safe_optimize: bool = True, optimize_with_info: bool = True
     ) -> None:
         self.pipeline = pipeline
@@ -165,7 +165,7 @@ class Optimize(BaseOptimize[OptimizablePipelineT, DatasetT]):
         Parameters
         ----------
         dataset
-            An instance of a :class:`~tpcp.dataset.Dataset` containing one or multiple data points that can
+            An instance of a :class:`~tpcp.Dataset` containing one or multiple data points that can
             be used for optimization.
             The structure of the data and the available reference information will depend on the dataset.
         optimize_params
@@ -259,7 +259,7 @@ class GridSearch(BaseOptimize[PipelineT, DatasetT], Generic[PipelineT, DatasetT,
     ----------
     gs_results_
         A dictionary summarizing all results of the gridsearch.
-        The format of this dictionary is designed to be directly passed into the :class:`~pd.DataFrame` constructor.
+        The format of this dictionary is designed to be directly passed into the :class:`~pandas.DataFrame` constructor.
         Each column then represents the result for one set of parameters
 
         The dictionary contains the following entries:
@@ -311,7 +311,7 @@ class GridSearch(BaseOptimize[PipelineT, DatasetT], Generic[PipelineT, DatasetT,
     best_score_: float
     multimetric_: bool
 
-    def __init__(  # noqa: super-init-not-called
+    def __init__(
         self,
         pipeline: PipelineT,
         parameter_grid: ParameterGrid,
@@ -396,7 +396,7 @@ class GridSearch(BaseOptimize[PipelineT, DatasetT], Generic[PipelineT, DatasetT,
 
         return self
 
-    def _format_results(self, candidate_params, out):  # noqa: no-self-use
+    def _format_results(self, candidate_params, out):
         """Format the final result dict.
 
         This function is adapted based on sklearn's `BaseSearchCV`
@@ -406,11 +406,14 @@ class GridSearch(BaseOptimize[PipelineT, DatasetT], Generic[PipelineT, DatasetT,
 
         results = {}
 
-        scores_dict = _normalize_score_results(out["scores"])
-        single_scores_dict = _normalize_score_results(out["single_scores"])
+        scores_dict = _normalize_score_results(out["scores"]) or {}
+        single_scores_dict = _normalize_score_results(out["single_scores"]) or {}
         for c, v in scores_dict.items():
             results[c] = v
             results[f"rank_{c}"] = np.asarray(rankdata(-v, method="min"), dtype=np.int32)
+        for c, _v in single_scores_dict.items():
+            # Because of custom aggregators, it can be that single scores dicts have different keys than the aggregated
+            # scores.
             results[f"single_{c}"] = single_scores_dict[c]
 
         results["data_labels"] = out["data_labels"]
@@ -491,7 +494,7 @@ class GridSearchCV(BaseOptimize[OptimizablePipelineT, DatasetT], Generic[Optimiz
         However, setting it incorrectly can lead detect errors that are very hard to detect in the final results.
 
         Instead of passing a list of names, you can also just set the value to `True`.
-        In this case all parameters of the provided pipeline that are marked as :func:`~tpcp.pure_parameter` are used.
+        In this case all parameters of the provided pipeline that are marked as :class:`~tpcp.PureParameter` are used.
         Note that pure parameters of nested objects are not considered, but only top-level attributes.
         If you need to mark nested parameters as pure, use the first method and pass the names (with `__`) as part of
         the list of names.
@@ -536,7 +539,7 @@ class GridSearchCV(BaseOptimize[OptimizablePipelineT, DatasetT], Generic[Optimiz
     ----------
     cv_results_
         A dictionary summarizing all results of the gridsearch.
-        The format of this dictionary is designed to be directly passed into the `pd.DataFrame` constructor.
+        The format of this dictionary is designed to be directly passed into the :class:`~pandas.DataFrame` constructor.
         Each column then represents the result for one set of parameters.
 
         The dictionary contains the following entries:
@@ -621,7 +624,7 @@ class GridSearchCV(BaseOptimize[OptimizablePipelineT, DatasetT], Generic[Optimiz
     multimetric_: bool
     final_optimize_time_: float
 
-    def __init__(  # noqa: super-init-not-called
+    def __init__(
         self,
         pipeline: OptimizablePipelineT,
         parameter_grid: ParameterGrid,
@@ -652,9 +655,7 @@ class GridSearchCV(BaseOptimize[OptimizablePipelineT, DatasetT], Generic[Optimiz
         self.safe_optimize = safe_optimize
         self.optimize_with_info = optimize_with_info
 
-    def optimize(
-        self, dataset: DatasetT, *, groups=None, mock_labels=None, **optimize_params
-    ) -> Self:  # noqa: arguments-differ
+    def optimize(self, dataset: DatasetT, *, groups=None, mock_labels=None, **optimize_params) -> Self:
         """Run the GridSearchCV on the given dataset.
 
         Parameters
@@ -765,7 +766,7 @@ class GridSearchCV(BaseOptimize[OptimizablePipelineT, DatasetT], Generic[Optimiz
 
         return self
 
-    def _format_results(self, candidate_params, n_splits, out, more_results=None):  # noqa: MC0001
+    def _format_results(self, candidate_params, n_splits, out, more_results=None):  # noqa: C901
         """Format the final result dict.
 
         This function is adapted based on sklearn's `BaseSearchCV`.
@@ -846,18 +847,23 @@ class GridSearchCV(BaseOptimize[OptimizablePipelineT, DatasetT], Generic[Optimiz
         # Store a list of param dicts at the key 'params'
         results["params"] = candidate_params
 
-        test_scores_dict = _normalize_score_results(out["test_scores"])
-        test_single_scores_dict = _normalize_score_results(out["test_single_scores"])
+        test_scores_dict = _normalize_score_results(out["test_scores"]) or {}
+        test_single_scores_dict = _normalize_score_results(out["test_single_scores"]) or {}
 
         for scorer_name in test_scores_dict:
             # Computed the (weighted) mean and std for test scores alone
             _store(f"test_{scorer_name}", test_scores_dict[scorer_name], splits=True, rank=True, weights=None)
+        for scorer_name in test_single_scores_dict:
+            # Because of custom aggregators, it can be that single scores dicts have different keys than the aggregated
+            # scores.
             _store_non_numeric(f"test_single_{scorer_name}", test_single_scores_dict[scorer_name])
-            if self.return_train_score:
-                train_scores_dict = _normalize_score_results(out["train_scores"])
-                train_single_scores_dict = _normalize_score_results(out["train_single_scores"])
 
+        if self.return_train_score:
+            train_scores_dict = _normalize_score_results(out["train_scores"])
+            train_single_scores_dict = _normalize_score_results(out["train_single_scores"])
+            for scorer_name in train_scores_dict:
                 _store(f"train_{scorer_name}", train_scores_dict[scorer_name], splits=True)
+            for scorer_name in train_single_scores_dict:
                 _store_non_numeric(f"train_single_{scorer_name}", train_single_scores_dict[scorer_name])
 
         return results
